@@ -21,7 +21,9 @@ if {[info exists gurkmeja]} {
 
 set ::kG [lsort [info globals]]
 set ::kC [info commands]
-foreach pat {{tcl::[a-z]*} {msgcat::[a-z]*} {oo::[a-z]*}} {
+set ::kP [list msgcat Tcl]
+
+foreach pat {{tcl::[a-z]*}} {
     foreach p [info commands $pat] {
         if {[string match ::* $p]} {
             set p [string range $p 2 end]
@@ -31,14 +33,21 @@ foreach pat {{tcl::[a-z]*} {msgcat::[a-z]*} {oo::[a-z]*}} {
 }
 
 # Collect exported namespace commands
-if 0 { # Not working yet
+if 1 { # Not working yet?
     set todo [namespace children ::]
     while {[llength $todo] > 0} {
         set ns [lindex $todo 0]
         set todo [lrange $todo 1 end]
-        eval lappend todo [namespace children $ns]
+        # Do not recurse in these namespaces
+        if {[lsearch {::oo} $ns] < 0} {
+            eval lappend todo [namespace children $ns]
+        }
 
         set exports [namespace eval $ns {namespace export}]
+        if {[llength $exports] == 0} {
+            # Assume lowercase-convention for public commands
+            set exports [list {[a-z]*}]
+        }
         foreach pat $exports {
             foreach p [info commands ${ns}::$pat] {
                 # Do not include the first :: in the name
@@ -110,6 +119,10 @@ proc markCmdAsKnown {args} {
 
 # Build a syntax database and write it to a channel
 proc buildDb {ch} {
+    # Allow predefined info to be provided from outside
+    array set syntax [array get ::syntax]
+    array set getSubCmd [array get ::getSubCmd]
+
     set patch [info patchlevel]
     set ver   [package present Tcl]
 
@@ -123,6 +136,7 @@ proc buildDb {ch} {
         if {![catch {tk windowingsystem}]} {
             append dbstring " [tk windowingsystem]"
         }
+        lappend ::kP Tk
     }
 
     # Below is the hardcoded syntax for many core commands.
@@ -146,9 +160,18 @@ proc buildDb {ch} {
     # s  Subcommand
     # d  Definition. That arg defines a new command
     #    If followed by =cmd, it copies syntax from cmd.
+    # di Define inheritance
+    # dc Define with copy
+    # do Define object
+    # dk Define constructor (args+body)
+    # dp Define procedure (name+args+body)
+    # dm Define method (name+args+body)
+    # dmp Define metod/procedure (name+args+body)
     # e  Expression
     # E  Expression that should be in braces
     # c  Code, checked in surrounding context
+    #    If an integer is added to it, that number of argumenst are added to the
+    #    code to emulate a command prefix. (cg has this too)
     # cg Code, checked in global context
     # cn Code, checked in virtual namespace
     # cl Code, checked in its own local context
@@ -158,6 +181,7 @@ proc buildDb {ch} {
     # n  The variable does not have to exist, and is set by the command.
     # v  The variable must exist. It is not marked as set.
     # l  Does not have to exist. It will be marked as known, but not set.
+    # nl Like n, but a list of variable names
 
     # Modifiers that apply to some of the above
     # ? Zero or One
@@ -292,6 +316,8 @@ proc buildDb {ch} {
     set syntax(open)            "r 1 3"
     # "package" is handled specially
     set syntax(package)         "s x*" ;# FIXA: All subcommands
+    set syntax(package\ require) "o* x x*"
+    set option(package\ require) "-exact"
     set syntax(pid)             "r 0 1"
     set syntax(proc)            dp
     set syntax(puts)            "1: x : o? x x?"
@@ -337,6 +363,8 @@ proc buildDb {ch} {
     set syntax(trace)           "s x x*"
     set syntax(trace\ add)      "s x x x"
     set syntax(trace\ add\ command)   "x x c3"
+    # FIXA: The second arg to trace add execution is not a sub command really,
+    # since it allows a list of operations. How to handle this?
     set syntax(trace\ add\ execution) "x s c2"
     set syntax(trace\ add\ execution\ leave) "c4"
     set syntax(trace\ add\ execution\ leavestep) "c4"
@@ -370,10 +398,9 @@ proc buildDb {ch} {
         set syntax(dict\ set)     "n x x*"
         set syntax(dict\ unset)   "n x x*"
         # FIXA: handle this style:
-        set syntax(dict\ update)  "n x x x* c"
-        set syntax(dict\ with)    "n x* c"
-        # FIXA: handle variables in dict for
-        set syntax(dict\ for)     "x x c"
+        set syntax(dict\ update)  "l x x x* c"
+        set syntax(dict\ with)    "l x* c"
+        set syntax(dict\ for)     "nl x c"
         
         # Initialising incr
         set syntax(incr)       "n x?"
@@ -431,16 +458,23 @@ proc buildDb {ch} {
         set syntax(chan\ pipe)               0
         set syntax(chan\ pop)                "x"
         set syntax(chan\ push)               "x c"
+        set syntax(dict\ map)                "nl x c"
         set syntax(file\ tempfile)           "n? x?*"
         set syntax(info\ coroutine)          0
         set syntax(interp\ cancel)           "o* x? x?"
         # New commands
-        set syntax(try)          "r 0"
+        # "try" is handled specially, but is added here to not disturb header
+        # gen, and to signal in to the engine that "try" is an allowed command.
+        set syntax(try)          "c x*"
+        set special(try) 1
         set syntax(throw)        "2"
         set syntax(coroutine)    "x x x*"
         set special(tailcall) 1
         set special(next) 1
         set syntax(yield)        "x?"
+        set syntax(yieldto)        "x x*"
+        set syntax(lmap) "n x c"
+        #set special(lmap) 1 FIXA
         set syntax(zlib)             "s x*"
         set syntax(zlib\ adler32)    "x x?"
         set syntax(zlib\ compress)   "x x?"
@@ -450,8 +484,15 @@ proc buildDb {ch} {
         set syntax(zlib\ gzip)       "x p*"
         set syntax(zlib\ gunzip)     "x p*"
         set syntax(zlib\ inflate)    "x x?"
-        set syntax(zlib\ push)       "s x*"
+        set syntax(zlib\ push)       "s x p*"
         set syntax(zlib\ stream)     "s x*"
+        set syntax(zlib\ stream\ compress)   "p*"
+        set syntax(zlib\ stream\ decompress) "p*"
+        set syntax(zlib\ stream\ deflate)    "p*"
+        set syntax(zlib\ stream\ gunzip)     "0" ;# No options for gunzip
+        set syntax(zlib\ stream\ gzip)       "p*"
+        set syntax(zlib\ stream\ inflate)    "p*"
+        lappend ::kP zlib
         set syntax(tcl::prefix)  "s x*"
         set syntax(tcl::prefix\ all)  "x x"
         set syntax(tcl::prefix\ longest)  "x x"
@@ -464,6 +505,17 @@ proc buildDb {ch} {
         set syntax(oo::class\ create::superclass)  di ;# Define inheritance
         set syntax(oo::class\ create::method) "dm"    ;# Define method 
         set syntax(oo::class\ create::destructor) c
+        set syntax(oo::class\ create::export) x
+        set syntax(oo::class\ create::class) x
+        set syntax(oo::class\ create::deletemethod) "x x*"
+        set syntax(oo::class\ create::export) "x x*"
+        set syntax(oo::class\ create::filter) "o? x*"
+        set syntax(oo::class\ create::forward) "x x x*"
+        set syntax(oo::class\ create::mixin) "o? x*"
+        set syntax(oo::class\ create::renamemethod) "x x"
+        set syntax(oo::class\ create::self) "x*"
+        set syntax(oo::class\ create::unexport) "x x*"
+
         set syntax(_stdclass_oo) "s x*"
         set subCmd(_stdclass_oo) "create new destroy variable"
         set syntax(_stdclass_oo\ create) "dc=_obj,_stdclass_oo x?"
@@ -475,6 +527,7 @@ proc buildDb {ch} {
         set syntax(info\ object) "s x x*"
         set syntax(info\ class)  "s x x*"
         set syntax(oo::copy)     "x x?"
+        lappend ::kP TclOO
 
         # FIXA: all oo::define
         set syntax(oo::define)   "2: x cn : x s x x*"
@@ -500,7 +553,7 @@ proc buildDb {ch} {
         # Set up basic checking of self/my
         set syntax(my)           "s x*"
         set syntax(my\ variable) "n*"
-        set syntax(self)         "s?"
+        set syntax(self)         "s"
         lappend ::kC self my
         oo::class create miffo {
             constructor {} {
@@ -535,7 +588,9 @@ proc buildDb {ch} {
         set syntax(bind)     "x x? cg?"
         set syntax(bindtags) "x x?"
         set syntax(clipboard) "s x*"
+        # console is windows only, but keep it in the db on all platforms
         set syntax(console)  "r 1"
+        set keep(console) 1
         set syntax(destroy)  "x*"
         set syntax(event)    "s x*"
         set syntax(focus)    "o? x?"
@@ -602,6 +657,7 @@ proc buildDb {ch} {
             set syntax(ttk::themes) x?
             set syntax(ttk::setTheme) x
             markCmdAsKnown ttk::style ttk::themes  ttk::setTheme
+            lappend ::kP Ttk tile
         }
         foreach class $classCmds {
             destroy .w
@@ -640,8 +696,8 @@ proc buildDb {ch} {
 
     # subCmd(cmd) contains a list of all allowed subcommands
 
-    # Get subcommands for commands that can't use the standard loop below
-    set subCmd(wm) [getSubCmds wm gurkmeja .]
+    # Get subcommands for commands that can't use the standard below
+    set getSubCmd(wm) "wm gurkmeja ."
 
     # Get subcommands for any commands defining "s"
     foreach cmd [array names syntax] {
@@ -655,7 +711,15 @@ proc buildDb {ch} {
             if {$ci >= 0 && $ci < $oi} {
                 set syn [lrange $syn [expr {$ci + 1}] end]
             }
-            set opts [eval getSubCmds $cmd $syn]
+            # Remove anything optional
+            set syn [lsearch -all -inline -not -regexp $syn {\*$}]
+
+            if {[info exists getSubCmd($cmd)]} {
+                set cmdToTry $getSubCmd($cmd)
+            } else {
+                set cmdToTry "$cmd $syn"
+            }
+            set opts [eval getSubCmds $cmdToTry]
             if {[llength $opts] > 0} {
                 set subCmd($cmd) $opts
                 #puts "AutoSub: $cmd $subCmd($cmd)"
@@ -675,6 +739,8 @@ proc buildDb {ch} {
     set option(unset)      [list -nocomplain --]
     set option(clock\ format) [getSubCmds clock format 1 -gurkmeja x]
 
+    set option(zlib\ push) [getSubCmds zlib push compress stdout -gurkmeja x]
+
     # Add additonal fconfigure, known for serial channels
     lappend option(fconfigure) -mode -handshake -queue -timeout -ttycontrol -ttystatus -xchar -pollinterval -sysbuffer -lasterror
     # For socket channels
@@ -742,7 +808,7 @@ proc buildDb {ch} {
         if {[string match "-*" [lindex $item end]]} {
             set opt [lindex $item end]
             set cmd [lrange $item 0 end-1]
-            if {[lsearch -exact $option($cmd) $opt] < 0} {
+            if {![info exists option($cmd)] || [lsearch -exact $option($cmd) $opt] < 0} {
                 #puts "Deleting option($item)"
                 unset option($item)
             }
@@ -765,8 +831,9 @@ proc buildDb {ch} {
     # Output the data
     puts $ch [list lappend ::dbInfo $dbstring]
     puts $ch [list set ::dbTclVersion $::tcl_version]
-    puts $ch [list set ::knownGlobals $::kG]
+    puts $ch [list set ::knownGlobals [lsort $::kG]]
     puts $ch [list set ::knownCommands [lsort $::kC]]
+    puts $ch [list set ::knownPackages [lsort $::kP]]
 
     foreach a {syntax return subCmd option} {
         foreach i [lsort [array names $a]] {
@@ -775,7 +842,8 @@ proc buildDb {ch} {
                 set first [lindex [split $i] 0]
                 if {![string match _* $first] && \
                         ![string match *::* $first] && \
-                        [lsearch $::kC $first] == -1} {
+                        [lsearch $::kC $first] == -1 && \
+                        ![info exists keep($first)]} {
                     puts stderr "Skipping ${a}($i) since $i is not known."
                 } else {
                     puts $ch [list set ::${a}($i) $v]
