@@ -1,6 +1,6 @@
 #!/bin/sh
 # the next line restarts using tclsh \
-exec tclsh8.5 "$0" ${1+"$@"}
+exec /usr/local/bin/tclsh8.5 "$0" ${1+"$@"}
 
 package require Tk
 if {[catch {package require textSearch}]} {
